//
// Copyright (c) 2012, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Windows.Forms;

using MindFusion.Layout;
using MindFusion.Diagramming.Lanes;
using MindFusion.Diagramming.Layout;

using Pen = MindFusion.Drawing.Pen;
using SolidBrush = MindFusion.Drawing.SolidBrush;


namespace MindFusion.Diagramming.WinForms.Samples.CS.ProcessLayout
{
	/// <summary>
	/// Summary description for MainForm.
	/// </summary>
	public partial class MainForm : Form
	{
		public MainForm()
		{
			InitializeComponent();
		}

		private void _loadDiagramButton_Click(object sender, EventArgs e)
		{
			Init();
			LoadFile();
		}

		private void Init()
		{
			random = new Random(DateTime.Now.Second);
		}

		private void MainForm_Load(object sender, EventArgs e)
		{
			Init();
			LoadFile();
			diagramView.ShowToolTips = true;
			diagramView.ToolTipDelay = 0;
		}

		private void LoadFile()
		{
			diagram.LoadFromFile("Process.fc");

			foreach (DiagramNode node in diagram.Nodes)
			{
				PointF position = new PointF(
					random.Next((int)(diagram.Bounds.X), (int)(diagram.Bounds.Right)),
					random.Next((int)(diagram.Bounds.Y), (int)(diagram.Bounds.Bottom)));
				SizeF size = new SizeF(node.Bounds.Width, node.Bounds.Height);

				node.Bounds = new RectangleF(position, size);
				InitNode(node);
			}

			foreach (DiagramLink link in diagram.Links)
			{
				link.Brush = new SolidBrush(Color.LightSkyBlue);
				link.Pen = new Pen(Color.DarkBlue);
				link.HeadPen = new Pen(Color.DarkBlue);
				link.HeadShape = ArrowHeads.Triangle;
			}

			diagram.LaneGrid[null, null].Style.BottomBorderPen = new Pen(Color.AliceBlue);
			diagram.LaneGrid[null, null].Style.LeftBorderPen = new Pen(Color.AliceBlue);
			diagram.LaneGrid[null, null].Style.RightBorderPen = new Pen(Color.AliceBlue);
			diagram.LaneGrid[null, null].Style.TopBorderPen = new Pen(Color.AliceBlue);
		}

		private void InitNode(DiagramNode node)
		{
			node.Brush = new SolidBrush(Color.CornflowerBlue);
			node.Pen = new Pen(Color.Navy);
			node.ToolTip = "Lane number: " + node.LayoutTraits[SwimlaneLayoutTraits.Lane];
		}

		private void arrangeButton_Click(object sender, EventArgs e)
		{
			// apply SwimlaneLayout
			SwimlaneLayout layout = new SwimlaneLayout();
			layout.Anchoring = Anchoring.Reassign;
			layout.NodeDistance = 20;
			layout.LaneDistance = 15;
			layout.Arrange(diagram);

			diagram.ResizeToFitItems(30);

			// zoom to show the whole diagram
			RectangleF r = diagram.Bounds;
			r.Inflate(5, 5);
			diagram.Bounds = r;
			diagramView.ZoomToRect(r);
		}

		private void diagram_NodeCreated(object sender, NodeEventArgs e)
		{
			// assign the new node to the swimlane that contains its center
			e.Node.LayoutTraits[SwimlaneLayoutTraits.Lane] = 0;

			RectangleF cellBounds = new RectangleF();
			ICell cell = diagram.LaneGrid.GetCellFromPoint(e.Node.GetCenter(), ref cellBounds);
			if (cell != null)
			{
				int index = diagram.LaneGrid.GetCellRow(cell);
				e.Node.LayoutTraits[SwimlaneLayoutTraits.Lane] = index;
			}

			InitNode(e.Node);
		}


		private Random random = new Random((int)DateTime.Now.Ticks);
	}
}